#####################################################################################
#   Title: Atomistic modelling and NMR studies reveal that gallium can target the   #
#          ferric PQS uptake system in P. aeruginosa biofilms                       #
#                                                                                   #
#   Authors: Oliver J. Hills (1),  Isaac Noble (1), Alex Heyam (2),                 #
#            Anthony Buckley (1), Andrew J. Scott (3), James Smith (1) &            #
#            Helen F. Chappell (1*)                                                 #
#                                                                                   #
#   (1) School of Food Science & Nutrition, University of Leeds, Woodhouse Lane,    #
#       Leeds, LS2 9JT, UK                                                          #
#   (2) School of Chemistry, University of Leeds, Woodhouse Lane, Leeds, LS2 9JT,   #
#       UK                                                                          #
#   (3) School of Chemical & Process Engineering, University of Leeds,              #
#       Woodhouse Lane, Leeds, LS2 9JT, UK                                          #
#                                                                                   #
#   * Corresponding author                                                          #
#   Email: H.F.Chappell@leeds.ac.uk (HFC)                                           #
#####################################################################################

#####################################################################################
#                      SCRIPT FOR CALCULATING FORMATION ENERGIES                    #
#                                Written by Isaac Noble                             #                    
#####################################################################################

# Set Up ----

# Set working directory.
setwd("*/Dataset/")
# * = Location of Dataset Directory

# Load packages.
library(tidyverse) 
# For data manipulation
# Use "install.packages("tidyverse")" to install it.


# Calculating the Formation Energy for Initial [Fe(PQS)3] ----

# Read in the CASTEP file for [Fe(PQS)3] and isolate the dispersion-corrected final 
# free energy value of the optimised structure. Call this object "E_Fe_PQS3".
E_Fe_PQS3 <- read_lines("Data/GO_Fe_PQS3_Outputs/FePQS_HighSpin_950.castep") %>% 
             # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  filter(str_detect(., "Dispersion corrected final free energy")) %>% 
  # isolate rows containing dispersion-corrected final free energy values
  tail(., n = 1) %>% 
  # isolate row with the last dispersion-corrected final final free energy value
  str_extract(., "-\\d+.\\d+") %>% 
  # extract the actual dispersion-corrected final energy value
  as.numeric()
  # convert number from "character" to "numeric" class

# Read in the CASTEP file for a single PQS chain and isolate the dispersion-
# corrected final free energy value of the optimised structure. Call this object 
# "E_PQS".
E_PQS <- read_lines("Data/GO_PQS_Outputs/oj_pqs.castep") %>% 
         # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  filter(str_detect(., "Dispersion corrected final free energy")) %>% 
  # isolate rows containing dispersion-corrected final free energy values
  tail(., n = 1) %>% 
  # isolate row with the last dispersion-corrected final final free energy value
  str_extract(., "-\\d+.\\d+") %>% 
  # extract the actual dispersion-corrected final energy value
  as.numeric()
  # convert number from "character" to "numeric" class

# Read in the CASTEP file for the body-centred cubic, alpha crystal of Fe and 
# isolate the final free energy value of the optimised structure. Call this object 
# "U_Fe_Prep".
U_Fe_Prep <- read_lines("Data/GO_Fe_BCC_A_Outputs/fe_bcc_666.castep") %>% 
  # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  filter(str_detect(., "Final free energy")) %>% 
  # isolate rows containing final free energy values
  tail(., n = 1) %>% 
  # isolate row with the last final final free energy value
  str_extract(., "-\\d+.\\d+") %>% 
  # extract the actual final energy value
  as.numeric() 
  # convert number from "character" to "numeric" class

# Divide U_Fe_Prep by 2 (no. atoms per unit cell of the crystal) to obtain the 
# chemical potential for Fe. Call this object "U_Fe".
U_Fe <- U_Fe_Prep / 2

# The equation for calculating the formation energy of [Fe(PQS)3] from its 
# constituents ("in situ" formation) is given by: 
# Ef = E_Fe_PQS3 - ((3 * E_PQS) + U_Fe)
# Within this  equation, E_Fe_PQS3  represents the energy of the [Fe(PQS)3] system, 
# E_PQS represents the energy of the PQS molecule that has been modified to remove 
# the hydrogen atom from its OH functional group, and U_Fe represents the chemical 
# potential of iron.

# Using E_Fe_PQS3, E_PQS, and U_Fe, calculate the formation energy of [Fe(PQS3)] 
# made from its constituents. Call this object "Ef".
Ef <- E_Fe_PQS3 - ((3 * E_PQS) + U_Fe)

# Make a table showing the above calculation.
Row1 <- "Ef = E_Fe_PQS3 - ((3 * E_PQS) + U_Fe)"
Row2 <- paste0("Ef = (", E_Fe_PQS3, ") - ((3 * (", E_PQS, ")) + (", U_Fe, ")")
Row3 <- paste0("Ef = (", E_Fe_PQS3, ") - ((", 3 * E_PQS, ") + (", U_Fe, ")")
Row4 <- paste0("Ef = (", E_Fe_PQS3, ") - (", 3 * E_PQS  +  U_Fe, ")")
Row5 <- paste0("Ef = ", E_Fe_PQS3 - ((3 * E_PQS) + U_Fe), " eV")
Ef_Table_Fe_PQS3 <- rbind(Row1, Row2, Row3, Row4, Row5)
# binds rows together into a table
colnames(Ef_Table_Fe_PQS3) <- "Formation_Energy_Calculation_Fe_PQS3"
# changes column of Ef_Table_Fe_PQS3 to text given
rownames(Ef_Table_Fe_PQS3) <- NULL
# removes row names

# Save the formation energy calculation table to the Data/ directory.
write.table(x = Ef_Table_Fe_PQS3, file = "Data/Formation_Energies/Ef_Fe_PQS3.txt")


# Calculating the Formation Energy for in situ [Ga(PQS)3] ----

# Read in the CASTEP file for [Ga(PQS)3] and isolate the dispersion-corrected final 
# free energy value of the optimised structure. Call this object "E_Ga_PQS3".
E_Ga_PQS3 <- read_lines("Data/GO_Ga_PQS3_Outputs/Fe_for_Ga_PQS.castep") %>% 
             # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  filter(str_detect(., "Dispersion corrected final free energy")) %>% 
  # isolate rows containing dispersion-corrected final free energy values
  tail(., n = 1) %>% 
  # isolate row with the last dispersion-corrected final final free energy value
  str_extract(., "-\\d+.\\d+") %>% 
  # extract the actual dispersion-corrected final energy value
  as.numeric()
  # convert number from "character" to "numeric" class

# Read in the CASTEP file for the orthorhombic, alpha crystal of Ga and isolate the 
# final free energy value of the optimised structure. Call this object "U_Ga_Prep".
U_Ga_Prep <- read_lines("Data/GO_Ga_ORH_A_Outputs/ga.castep") %>% 
             # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  filter(str_detect(., "Final free energy")) %>% 
  # isolate rows containing final free energy values
  tail(., n = 1) %>% 
  # isolate row with the last final final free energy value
  str_extract(., "-\\d+.\\d+") %>% 
  # extract the actual final energy value
  as.numeric() 
  # convert number from "character" to "numeric" class

# Divide U_Ga_Prep by 8 (no. atoms per unit cell of the crystal) to obtain the 
# chemical potential for Ga. Call this object "U_Ga".
U_Ga <- U_Ga_Prep / 8

# The equation for calculating the formation energy of [Ga(PQS)3] from its 
# constituents ("in situ" formation) is given by: 
# Ef = E_Ga_PQS3 - ((3 * E_PQS) + U_Ga)
# Within this  equation, E_Ga_PQS3  represents the energy of the [Ga(PQS)3] system, 
# E_PQS represents the energy of the PQS molecule that has been modified to remove 
# the hydrogen atom from the  OH functional group, and U_Ga represents the chemical
# potential of gallium.

# Using E_Ga_PQS3, E_PQS (from earlier), and U_Ga, calculate the formation energy of 
# [Ga(PQS3)] made from its constituents. Call this object "Ef".
Ef <- E_Ga_PQS3 - ((3 * E_PQS) + U_Ga)

# Make a table showing the above calculation.
Row1 <- "Ef = E_Ga_PQS3 - ((3 * E_PQS) + U_Ga)"
Row2 <- paste0("Ef = (", E_Ga_PQS3, ") - ((3 * (", E_PQS, ")) + (", U_Ga ,")")
Row3 <- paste0("Ef = (", E_Ga_PQS3 , ") - ((", 3 * E_PQS, ") + (", U_Ga, ")")
Row4 <- paste0("Ef = (", E_Ga_PQS3, ") - (", 3 * E_PQS  +  U_Ga, ")")
Row5 <- paste0("Ef = ", E_Ga_PQS3 - ((3 * E_PQS) + U_Ga), " eV")
Ef_Table_Ga_PQS3 <- rbind(Row1, Row2, Row3, Row4, Row5)
            # binds rows together into a table
colnames(Ef_Table_Ga_PQS3) <- "Formation_Energy_Calculation_Ga_PQS3"
# changes column of Ef_Table_Ga_PQS3 to text given
rownames(Ef_Table_Ga_PQS3) <- NULL
# removes row names

# Save the formation energy calculation table to the Data/ directory.
write.table(x = Ef_Table_Ga_PQS3, file = "Data/Formation_Energies/Ef_Ga_PQS3.txt")
        

# Feasibility for an iron-for-gallium cation exchange into [Fe(PQS)3] ----

# The equation for calculating the formation energy of [Ga(PQS)3] from an 
# iron-for-gallium cation exchange into [Fe(PQS)3] is given by:
# Ef = E_Ga_PQS3 - (E_Fe_PQS3 + U_Ga - U_Fe)
# Within this equation, E_Ga_PQS3  represents the energy of [Ga(PQS)3], E_Fe_PQS3  
# represents the energy of the initial [Fe(PQS)3] complex, U_Ga represents the 
# chemical potential of gallium, and U_Fe the chemical potential of iron.

# Using E_Ga_PQS3, E_Fe_PQS3, U_Ga, and U_Fe, calculate the formation energy of an 
# iron-for-gallium transition into [Fe(PQS3)] Call this object "Ef".
Ef <- E_Ga_PQS3 - (E_Fe_PQS3 + U_Ga - U_Fe)

# Make a table showing the above calculation.
Row1 <- "Ef = E_Ga_PQS3 - (E_Fe_PQS3 + U_Ga - U_Fe)"
Row2 <- paste0("(",E_Ga_PQS3,") - ((",E_Fe_PQS3,") + (",U_Ga,") - (",U_Fe,")")
Row3 <- paste0("(",E_Ga_PQS3,") - (",E_Fe_PQS3 + U_Ga - U_Fe,")")
Row4 <- paste0("Ef = ",E_Ga_PQS3 - (E_Fe_PQS3 + U_Ga - U_Fe)," eV")
Ef_Table_Fe4Ga <- rbind(Row1, Row2, Row3, Row4)
# binds rows together into a table
colnames(Ef_Table_Fe4Ga) <- "Formation_Energy_Calculation_Fe4Ga_Transition"
# changes column of Ef_Table_Fe4Ga to text given
rownames(Ef_Table_Fe4Ga) <- NULL
# removes row names

# Save the formation energy calculation table to the Data/ directory.
write.table(x    = Ef_Table_Fe4Ga, 
            file = "Data/Formation_Energies/Ef_Fe4Ga_Transition.txt")

