#####################################################################################################################################
#   Title: Atomistic modelling and NMR studies reveal that gallium can target the ferric PQS uptake system in P. aeruginosa         #
#          biofilms                                                                                                                 #
#                                                                                                                                   #
#   Authors: Oliver J. Hills (1),  Isaac Noble (1), Alex Heyam (2), Anthony Buckley (1), Andrew J. Scott (3), James Smith (1)       #
#            & Helen F. Chappell (1*)                                                                                               #
#                                                                                                                                   #
#   (1) School of Food Science & Nutrition, University of Leeds, Woodhouse Lane, Leeds, LS2 9JT, UK                                 #
#   (2) School of Chemistry, University of Leeds, Woodhouse Lane, Leeds, LS2 9JT, UK                                                #
#   (3) School of Chemical & Process Engineering, University of Leeds, Woodhouse Lane, Leeds, LS2 9JT, UK                           #
#                                                                                                                                   #
#   * Corresponding author                                                                                                          #
#   Email: H.F.Chappell@leeds.ac.uk (HFC)                                                                                           #
#####################################################################################################################################

#####################################################################################################################################
#                                                    SCRIPT FOR ISOLATING BOND POPULATIONS                                          #
#                                                            Written by Isaac Noble                                                 #                    
#####################################################################################################################################

# Set Up ----

# Set working directory.
setwd("*/Dataset/")
# * = Location of Dataset Directory

# Load packages.
library(tidyverse) 
# For data manipulation
# Use "install.packages("tidyverse")" to install it.
# Use "install.packages("janitor")" to obtain janitor::clean_names().


# Isolating Bond Populations of Geometry-Optimised [Fe(PQS)3] ----

# Read in the CASTEP file for [Fe(PQS)3]. Call it "Fe_PQS3_Raw".
Fe_PQS3_Raw  <- read_lines("Data/GO_Fe_PQS3_Outputs/FePQS_HighSpin_950.castep") %>% 
                # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  janitor::clean_names() 
  # column name change: "." -> "x"

# Isolate bond population header from "Fe_PQS3_Raw". Call this object "Fe_PQS3_BP_Start".
Fe_PQS3_BP_Start <- which(Fe_PQS3_Raw == "                 Bond                   Population        Spin       Length (A)") %>% 
                    # get row no. for specified row (bond population header)
  tibble()  
  # save row number to data frame ("bond")

# Isolate bond population data to "Fe_PQS3_BP".
Fe_PQS3_BP <- tail(Fe_PQS3_Raw, nrow(Fe_PQS3_Raw) - Fe_PQS3_BP_Start$. - 1) 

# Remove the tail of the bond population table.
Tail <- which(Fe_PQS3_BP == "================================================================================")
        # get row no. for specified row (bond population tail)
Fe_PQS3_BP_Only <- head(Fe_PQS3_BP, Tail - 1)

Fe_PQS3_BP_Cols <- Fe_PQS3_BP_Only %>%
  separate(x, c("Atom1", "Atom2"), sep = " -- ") %>% 
  separate(Atom2, c("Atom2", "Population"), sep = "                    |                  |                   ") %>% 
  separate(Population, c("Population", "Length_A"),  sep = "          ") %>% 
  separate(Population, c("Population", "Spin"), sep = "      ") %>% 
  # divides table into columns for each atom ("Atom1", "Atom2"), the bond population ("Population"), and bond length 
  # in angstroms ("Length_A")
  mutate(Atom1      = str_extract(Atom1, "[A-Z][a-z] \\d+|[A-Z] \\d+"),
         Population = str_extract(Population, "-0.\\d+|0.\\d+|1.\\d+"),
         Spin       = str_extract(Spin, "-0.\\d+|0.\\d+|1.\\d+"))
  # ensures only the values are kept, i.e., no spaces

# Incorporate information on bonding type to the table based on the bond population value. Call this new object 
# "Fe_PQS3_Interactions".
Fe_PQS3_Interactions <- Fe_PQS3_BP_Cols %>% 
  mutate(Interaction1 = ifelse(str_detect(Population, "^[+]?(0.4|0.5|0.6|0.7|0.8|0.9|1.0|1.1)"), "covalent", NA_character_),
         Interaction2 = ifelse(str_detect(Population, "^[+]?(0.1|0.2|0.3)"), "ionic", NA_character_),
         Interaction3 = ifelse(Population == "-0.00|0.00", "none", NA_character_),
         Interaction4 = ifelse(str_detect(Population, "^[+]?(0.0\\d)"), "very_weak", NA_character_),
         Interaction5 = ifelse(str_detect(Population, "-0.0\\d+"), "weak_repulsive", NA_character_),
         Interaction6 = ifelse(str_detect(Population, "-0.\\d|-0.\\d\\d"), "repulsive", NA_character_)) 
  # adds new columns that output names to an interaction if its value matches the regex given and NA everywhere else: 
  # Covalent (x > 0.4); Ionic/H-Bond (0.1 <= x < 0.4); Very Weak (0 < x < 0.1); None (x = 0); Weak Repulsive (0 > x > -0.1); 
  # Moderate Repulsive (-0.1 >= x > -val).

# Create a new column in Fe_PQS3_Interactions called Interaction that coalesces all the previous Interaction[1-6] columns into 
# one.
Fe_PQS3_Interactions$Interaction <- coalesce(Fe_PQS3_Interactions$Interaction1, Fe_PQS3_Interactions$Interaction2,
                                             Fe_PQS3_Interactions$Interaction3, Fe_PQS3_Interactions$Interaction4,
                                             Fe_PQS3_Interactions$Interaction5, Fe_PQS3_Interactions$Interaction6)
                                    # merge all interaction columns to "interaction"
Fe_PQS3_Interactions <- Fe_PQS3_Interactions %>% 
  select(-c(Interaction1, Interaction2, Interaction3, Interaction4, Interaction5, Interaction6)) 
  # removes all numbered interaction columns

# Save Fe_PQS3_Interactions to a text file.
write.table(Fe_PQS3_Interactions, file = "Data/Bond_Populations/Bond_Pop_Fe_PQS3.txt")

# Isolating Bond Populations of Geometry-Optimised [Ga(PQS)3] ----

# Read in the CASTEP file for [Ga(PQS)3]. Call it "Ga_PQS3_Raw".
Ga_PQS3_Raw  <- read_lines("Data/GO_Ga_PQS3_Outputs/Fe_for_Ga_PQS.castep") %>% 
                # one file line => one row of a table
  tibble() %>% 
  # all rows will belong to a column, "."
  janitor::clean_names() 
  # column name change: "." -> "x"

# Isolate bond population header from "Ga_PQS3_Raw". Call this object Ga_PQS3_BP_Start".
Ga_PQS3_BP_Start <- which(Ga_PQS3_Raw == "                 Bond                   Population      Length (A)") %>% 
                    # get row no. for specified row (bond population header)
  tibble()  
  # save row number to data frame ("bond")

# Isolate bond population data to "Ga_PQS3_BP".
Ga_PQS3_BP <- tail(Ga_PQS3_Raw, nrow(Ga_PQS3_Raw) - Ga_PQS3_BP_Start$. - 1) 

# Remove the tail of the bond population table.
Tail <- which(Ga_PQS3_BP == "======================================================================")
        # get row no. for specified row (bond population tail)
Ga_PQS3_BP_Only <- head(Ga_PQS3_BP, Tail - 1)

# Column-ify bond population data. Save object as "Ga_PQS3_BP_Cols".
Ga_PQS3_BP_Cols <- Ga_PQS3_BP_Only %>%
  separate(x, c("Atom1", "Atom2"), sep = " -- ") %>% 
  separate(Atom2, c("Atom2", "Population"), sep = "                    |                  |                   ") %>% 
  separate(Population, c("Population", "Length_A"),  sep = "        ") %>% 
  # divides table into columns for each atom ("Atom1", "Atom2"), the bond population ("Population"), and bond length 
  # in angstroms ("Length_A")
  mutate(Atom1      = str_extract(Atom1, "[A-Z][a-z] \\d+|[A-Z] \\d+"),
         Population = str_extract(Population, "-0.\\d+|0.\\d+|1.\\d+")) 
  # ensures only the values are kept, i.e., no spaces

# Incorporate information on bonding type to the table based on the bond population value. Call this new object
# "Ga_PQS3_Interactions".
Ga_PQS3_Interactions <- Ga_PQS3_BP_Cols %>% 
  mutate(Interaction1 = ifelse(str_detect(Population, "^[+]?(0.4|0.5|0.6|0.7|0.8|0.9|1.0|1.1)"), "covalent", NA_character_),
         Interaction2 = ifelse(str_detect(Population, "^[+]?(0.1|0.2|0.3)"), "ionic", NA_character_),
         Interaction3 = ifelse(Population == "-0.00|0.00", "none", NA_character_),
         Interaction4 = ifelse(str_detect(Population, "^[+]?(0.0\\d)"), "very_weak", NA_character_),
         Interaction5 = ifelse(str_detect(Population, "-0.0\\d+"), "weak_repulsive", NA_character_),
         Interaction6 = ifelse(str_detect(Population, "-0.\\d|-0.\\d\\d"), "repulsive", NA_character_)) 
  # adds new columns that output names to an interaction if its value matches the regex given and NA everywhere else: 
  # Covalent (x > 0.4); Ionic/H-Bond (0.1 <= x < 0.4); Very Weak (0 < x < 0.1); None (x = 0); Weak Repulsive (0 > x > -0.1); 
  # Moderate Repulsive (-0.1 >= x > -val).

# Create a new column in Ga_PQS3_Interactions called Interaction that coalesces all the previous Interaction[1-6] columns into 
# one.
Ga_PQS3_Interactions$Interaction <- coalesce(Ga_PQS3_Interactions$Interaction1, Ga_PQS3_Interactions$Interaction2,
                                             Ga_PQS3_Interactions$Interaction3, Ga_PQS3_Interactions$Interaction4,
                                             Ga_PQS3_Interactions$Interaction5, Ga_PQS3_Interactions$Interaction6)
                                    # merge all interaction columns to "interaction"
Ga_PQS3_Interactions <- Ga_PQS3_Interactions %>% 
  select(-c(Interaction1, Interaction2, Interaction3, Interaction4, Interaction5, Interaction6)) 
  # removes all numbered interaction columns

# Save Ga_PQS3_Interactions to a text file.
write.table(Ga_PQS3_Interactions, file = "Data/Bond_Populations/Bond_Pop_Ga_PQS3.txt")



